/*
	Hyperion - A network anomaly discovery system
	Copyright (C) 2011 Nicholas Tomlinson
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "config.h"
#include "error.hpp"

#include "assemblybuffer.hpp"

AssemblyBuffer::AssemblyBuffer()
{
	max = 0;
}

AssemblyBuffer::AssemblyBuffer(unsigned int pmax)
{
	max = pmax;
}

void AssemblyBuffer::AddBuffer(unsigned int offset, Buffer b)
{
	if (max && (offset + b.GetLength() > max)){
		throw FormatError("Cannot add buffer: would extend beyond maximum");
	}
	
	if (!b.GetLength()){
		throw FormatError("Cannot add buffer: zero length buffer");
	}
	
	for (std::map<unsigned int, Buffer>::iterator it = pieces.begin(); it != pieces.end(); it++){
		if ((offset < it->first + it->second.GetLength()) && (offset + b.GetLength() > it->first)){
			throw FormatError("Cannot add buffer: Overlap");
		}
	}
	
	pieces[offset] = b;
}

unsigned int AssemblyBuffer::PopMaxLength()
{
	unsigned int offset = 0;
	for (std::map<unsigned int, Buffer>::iterator it = pieces.begin(); it != pieces.end(); it++){
		if (it->first == offset){
			offset += it->second.GetLength();
		}
		else {
			return offset;
		}
	}
	return offset;
}

Buffer AssemblyBuffer::PopFirst()
{
	if (!pieces.count(0)){
		throw FormatError("Cannot pop first");
	}
	
	Buffer first = pieces[0];
	pieces.erase(0);
	std::map<unsigned int, Buffer> newpieces;
	for (std::map<unsigned int, Buffer>::iterator it = pieces.begin(); it != pieces.end(); it++){
		newpieces[it->first - first.GetLength()] = it->second;
	}
	pieces = newpieces;
	return first;
}

Buffer AssemblyBuffer::PopMax()
{
	Buffer b;
	unsigned int offset = 0;
	for (std::map<unsigned int, Buffer>::iterator it = pieces.begin(); it != pieces.end();){
		if (it->first == offset){
			offset += it->second.GetLength();
			b += it->second;
			pieces.erase(it++);
		}
		else {
			return b;
		}
	}
	return b;
}

void AssemblyBuffer::SetMax(unsigned int pmax)
{
	max = pmax;
}
