/*
	Hyperion - A network anomaly discovery system
	Copyright (C) 2011 Nicholas Tomlinson
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "config.h"
#include "error.hpp"

#include "config.h"
#include "error.hpp"

#include <string>

#include "configuration.hpp"
#include "configurationast.hpp"
#include "error.hpp"
#include "configurationparser.h"
#include "configurationlexer.h"

extern int yyparse(yyscan_t);

Configuration::Configuration(int argc, const char **argv)
{
	for (int i = 0; i < argc; i++){
		yyscan_t scanner;
		ParserData parserdata;
		
		parserdata.filename = argv[i];
		parserdata.scanner = &scanner;
		parserdata.error = false;
		
		yylex_init (&scanner);
		FILE *f = fopen(argv[i], "r");
		if (!f){
			throw FormatError("Could not open configuration file \"%s\"", argv[i]);
		}
		
		yyset_in(f, scanner);
		yyset_out(stderr, scanner);
		
		if (yyparse((void *)&parserdata)){
			throw FormatError("Could not parse configuration file \"%s\"", argv[i]);
		}
		
		yylex_destroy (scanner);
		fclose(f);
	
		if (parserdata.error){
			throw FormatError("Configuration parse error in file \"%s\"", argv[i]);
		}
		
		if (!parserdata.root){
			throw FormatError("Could not parse \"%s\"", argv[i]);
		}
		
		parserdata.root->Execute(root);
		delete parserdata.root;
	}
	
	root.SetRoKey();
	
	if (root["hyperion"]["debug"]){
		printf("%s\n\n", root.ToString().c_str());
	}
}

DynamicValue Configuration::operator[](std::string name)
{
	return root[name];
}
