/*
	Hyperion - A network anomaly discovery system
	Copyright (C) 2011-2012 Nicholas Tomlinson
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "config.h"
#include "error.hpp"

#ifndef EVENT_HPP
#define EVENT_HPP

#include <map>
#include <set>
#include <list>
#include <string>
#include <typeinfo>

#include "time.hpp"
#include "dynamic.hpp"

class Event
{
	public:
		Event(); //EmptyEvent
		virtual ~Event();
		
		Time GetTimestamp();
			//TODO: Make it possible to set the clock offset (or otherwise the clock absolutely)
			//This might seem like a bit of a hack, but actually, it enables a
			//replay sniffer to:
			//	a) make things like timers work:D (think about how to make this
			//		work where a timer should go off in the middle of a gap in
			//		packets)
			//	b) Avoids the need to pass the timestamp of an event around all
			//		the time
			//Also add the GPL headers at some point:p
		std::string GetFilter();
		std::string ToString();
		
		unsigned int Count();
		bool Has(std::string name);
		DynamicValue &operator[](std::string key);
		
	private:
		Time timestamp;
		DynamicValue root;
};

//There is a reason this isn't called "event handler", and that's because this
//is really meant for working out where events are supposed to be handled, not
//for actually handling them
//Though this might change if I make a way to chain these
class EventSubscription
{
	public:
		EventSubscription();
		virtual ~EventSubscription();
		
		virtual void HandleEvent(Event &event) = 0;
		
	protected:
		void IncRefCount();
		void DecRefCount();
		
	private:
		unsigned int refcount;
};

//library of subscription classes
#include "subscriptions.hpp"

class Modules;
template <typename KEY> class EventTable;
class EventManager
{
	public:
		EventManager(Modules *modules);
		virtual ~EventManager(); //This deletes all subscriptions
		
		//TODO: Consider removing the const requirement, and instead relying on
		//		the event methods to avoid bad modification
		//		This would enable various optimizations (such as cache some
		//		piece of information if and only if it's asked for)
		void Publish(Event event);
		
		//These are for convenience
		
		void Subscribe(EventSubscription *subscription);
		void Unsubscribe(EventSubscription *subscription); //NOTE: this deletes the subscription too
		EventTable<uint32_t> *GetIntEventTable(std::string name);
		
	private:
		EventManager(const EventManager &);
		EventManager &operator=(const EventManager &);
		
		Modules *modules;
		std::set<EventSubscription *> subscriptions; //we shoudl either change
			//this to EventSubscription & if possible, or possibly define that
			//~EventManager() and Unsubscribe deletes these?
		std::set<EventSubscription *> unsubscriptions;
		std::map<std::string, EventTable<uint32_t> *> inttables;
		std::list<Event> queue;
};
#include "eventtable.hpp"

#endif
