/*
	Hyperion - A network anomaly discovery system
	Copyright (C) 2011 Nicholas Tomlinson
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "config.h"
#include "error.hpp"

//I wouldn't expect this header to be included directly
#ifndef EVENT_HPP
#error Include event.hpp, not eventtable.hpp
#endif

#ifndef EVENTTABLE_HPP
#define EVENTTABLE_HPP

#include "config.h"
#include "error.hpp"

#include <set>
#include <map>

#include "buffer.hpp"
#include "modules.hpp"

//Perhaps make this more generic - an event receiver/sender (i.e. an event translator)
template <typename KEY> class EventTable
{
	public:
		EventTable(std::string pname, Modules *pmodules, EventManager *pmanager)
		{
			modules = pmodules;
			manager = pmanager;
			name = pname;
		}
		
		virtual ~EventTable(){}
		
		void Add(std::string name, KEY key)
		{
			names[key] = name;
		}
		
		void Publish(KEY key, Buffer buffer)
		{
			Event e;
			if (names.count(key)){
				e[names[key]]["data"] = buffer;
				manager->Publish(e);
			}
			else {
				Event e;
				std::string en = std::string("anomaly:table:") + name + std::string(":unknown");
				e[en]["key"] = key;
				e[en]["buffer"] = buffer;
				manager->Publish(e);
			}
		}
		
	private:
		std::string name;
		std::map<KEY, std::string> names;
		Modules *modules;
		EventManager *manager;
};

#endif
