/*
	Hyperion - A network anomaly discovery system
	Copyright (C) 2011 Nicholas Tomlinson
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "config.h"
#include "error.hpp"

#ifndef SOCKET_HPP
#define SOCKET_HPP

#include "event.hpp"
#include "module.hpp"
#include "time.hpp"

#include <map>

//TODO: wrap sockets up more nicely
class SocketModuleBase : public Module
{
	public:
		SocketModuleBase(const char *name, Modules *modules, EventManager *manager,
			Configuration *configuration, Database *database);
		
		virtual void HandleSocketEvent() = 0;
		
	protected:
		//we might also convert these into sending the new socket event its self?
		void AddSocket(int socket);
		void RemoveSocket(int socket);
		
	private:
		void OnSocketEvent(Event event);
};

class SocketModule : public Module
{
	public:
		SocketModule(Modules *modules, EventManager *manager, Configuration *configuration, Database *database);
		
		void ProcessSockets(Time t);
		
	private:
		void OnNewSocket(Event event);
		
		std::map<int, SocketModuleBase *> sockets;
};

#endif
