/*
	Hyperion - A network anomaly discovery system
	Copyright (C) 2011 Nicholas Tomlinson
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "config.h"
#include "error.hpp"

#include "event.hpp"
#include "timer.hpp"

TimerModule::TimerModule(Modules *modules, EventManager *manager, Configuration *configuration, Database *database) :
	Module("Timer", modules, manager, configuration, database)
{
	GetManager()->Subscribe(GetCallbackNameEventSubscription<TimerModule>
		("timer", this, &TimerModule::OnNewTimer));
}

Time TimerModule::ProcessTimers()
{
	Time now = Time::now();
	for (std::list<Timer>::iterator it = timers.begin(); it != timers.end();){
		if (it->t <= now){
			GetManager()->Publish(it->e);
			it = timers.erase(it);
		}
		else {
			it++;
		}
	}
	
	if (!timers.size()){
		return Time::s(0); //0 = no more timers
	}
	Time min = timers.front().t - now;
	for (std::list<Timer>::iterator it = timers.begin(); it != timers.end(); it++){
		if (it->t < min){
			min = it->t;
		}
	}
	
	return min;
}

void TimerModule::OnNewTimer(Event event)
{
	Timer timer;
	timer.e = event["timer"]["event"];
	timer.t = Time::now() + event["timer"]["time"];
	timers.push_back(timer);
}

//This has to go somewhere, so why not here?
const Time Time::zero;
