import struct

def Rfc1071(data):
	checksum = 0
	length = len(data)
	for i in range(0, int((length + 1)/2)):
		n = data[i*2]
		if (i * 2 != length - 1):
			n += data[(i*2) + 1] * 256
		checksum += n
		
	while (checksum & 0xFFFF0000) != 0:
		checksum = (checksum & 0xFFFF) + int(checksum / 65536);
	
	return (~checksum) & 0xFFFF

def MakeEthernetPacket(src, dst, ethertype, payload=None):
	def addr(s):
		if len(s) != 17:
			raise BaseException("Length of ethernet address is not 17: " + s)
		if (s[2] != ':') and (s[5] != ':') and (s[8] != ':') and (s[11] != ':') and (s[14] != ':'):
			raise BaseException("Invalid ethernet address: " + s)
		b0 = int(s[0:2], 16)
		b1 = int(s[3:5], 16)
		b2 = int(s[6:8], 16)
		b3 = int(s[9:11], 16)
		b4 = int(s[12:14], 16)
		b5 = int(s[15:17], 16)
		return struct.pack("BBBBBB", b0, b1, b2, b3, b4, b5)
	
	src = addr(src)
	dst = addr(dst)
	ethertype = struct.pack("!H", ethertype)
	
	if payload == None:
		return dst + src + ethertype
	else:
		return dst + src + ethertype + payload

def Ipv4Addr(s):
		ss = s.split(".")
		if len(ss) != 4:
			raise BaseException("Invalid IPv4 address: " + s)
		b0 = int(ss[0])
		b1 = int(ss[1])
		b2 = int(ss[2])
		b3 = int(ss[3])
		return struct.pack("BBBB", b0, b1, b2, b3)

def MakeIpv4Packet(src, dst, proto, payload=None, ttl=255, identification=0):
	src = Ipv4Addr(src)
	dst = Ipv4Addr(dst)
	length = 0
	if payload != None:
		length = len(payload)
	
	header = struct.pack("!BBHHHBBH", 0x45, 0, 20 + length, identification, 0, ttl, proto, 0) + src + dst
	checksum = Rfc1071(header)
	header = struct.pack("!BBHHHBBH", 0x45, 0, 20 + length, identification, 0, ttl, proto, checksum) + src + dst
	#header[10] =  & 0xFF
	#header[11] = int(checksum / 256)
	
	if payload == None:
		return header
	else:
		return header + payload

def MakeIpv4PseudoHeader(src, dst, proto, payload=None):
	src = Ipv4Addr(src)
	dst = Ipv4Addr(dst)
	length = 0
	if payload != None:
		length = len(payload)
	return src + dst + struct.pack("!BBH", 0, proto, length)

def MakeTcpPacket(pseudoheader, src, dst, payload=None, seqn=0, ackn=0, ws=0, urgp=0,
	fin=False, syn=False, rst=False, psh=False, ack=False, urg=False, ece=False, cwr=False, ns=False):
	data_offset = 5 * 16
	if ns:
		data_offset += 1
	
	flags = 0
	if cwr:
		flags += 128
	if ece:
		flags += 64
	if urg:
		flags += 32
	if ack:
		flags += 16
	if psh:
		flags = 8
	if rst:
		flags += 4
	if syn:
		flags += 2
	if fin:
		flags += 1
	
	header = struct.pack("!HHIIBBHHH", src, dst, seqn, ackn, data_offset, flags, ws, 0, urgp)
	if payload == None:
		checksum_data = pseudoheader + header
	else:
		checksum_data = pseudoheader + header + payload
	checksum = Rfc1071(checksum_data)
	header = struct.pack("!HHIIBBHHH", src, dst, seqn, ackn, data_offset, flags, ws, checksum, urgp)
	if payload == None:
		return header
	else:
		return header + payload

def MakeTcpIpv4EthernetPacket(esrc, isrc, tsrc, edst, idst, tdst, payload=None, seqn=0, ackn=0, ws=0, syn=False, ack=False):
	pseudoheader = MakeIpv4PseudoHeader(isrc, idst, 6, payload)
	tcp = MakeTcpPacket(pseudoheader, tsrc, tdst, payload, seqn, ackn, ws, syn=syn, ack=ack)
	ip = MakeIpv4Packet(isrc, idst, 6, tcp)
	ethernet = MakeEthernetPacket(esrc, edst, 0x800, ip)
	return ethernet

